/*
 * handlesignals.c
 *
 *  Created on: 22.06.2010
 *      Author: srr
 */

#include <config.h>
#include <sqlrconnection.h>

signalhandler *sqlrconnection_svr::handleSignals(void (*shutdownfunction)(int)) {

	// handle kill signals (SIGINT, SIGTERM)
	this->handleShutDown(shutdownfunction);

	// handle crash signals (SIGSEGV)
	// read or write outside the memory that is allocated

	// We don't want to catch SEGV, we want core to be dumped
	this->handleCrash(shutdownfunction);

	// but daemonprocess sets it's own handler for SIGSEGV
	// Since rudiments does not provide us with service to set
	// default signal handler, we should do it by direct call to signal(2)
	//signal (SIGSEGV, SIG_DFL);

	signalhandler *sighandler=new signalhandler();
	sighandler->setHandler(shutdownfunction);


	sighandler->handleSignal(SIGALRM); // handle alarm for timeouts

#ifdef HAVE_SIGQUIT
	sighandler->handleSignal(SIGQUIT);
#endif

	// handle program bugs
#ifdef HAVE_SIGBUS
	// This signal is generated when an invalid pointer is dereferenced.
	sighandler->handleSignal(SIGBUS);
#endif
#ifdef HAVE_SIGFPE
	// fatal arithmetic error
	sighandler->handleSignal(SIGFPE);
#endif
#ifdef HAVE_SIGILL
	// trying to execute garbage or a privileged instruction
	sighandler->handleSignal(SIGILL);
#endif
#ifdef HAVE_SIGABRT
	// calling abort
	sighandler->handleSignal(SIGABRT);
#endif
#ifdef HAVE_SIGIOT
	// Generated by the PDP-11 “iot” instruction. On most machines, this is just another name for SIGABRT
	sighandler->handleSignal(SIGIOT);
#endif
#ifdef HAVE_SIGTRAP
	// program will probably only see SIGTRAP if it is somehow executing bad instructions
	sighandler->handleSignal(SIGTRAP);
#endif
#ifdef HAVE_SIGEMT
	// unimplemented instructions which might be emulated in software
	sighandler->handleSignal(SIGEMT);
#endif
#ifdef HAVE_SIGSYS
	// Bad system call
	sighandler->handleSignal(SIGSYS);
#endif
#ifdef HAVE_SIGXCPU
	// CPU consumption "soft limit"
	sighandler->handleSignal(SIGXCPU);
#endif
#ifdef HAVE_SIGXFSZ
	// File size limit exceeded
	sighandler->handleSignal(SIGXFSZ);
#endif
#ifdef HAVE_SIGPWR
	// power failure
	sighandler->handleSignal(SIGPWR);
#endif

	// ignore others
	signalset	set;
	set.removeAllSignals();

	// HangUp
#ifdef HAVE_SIGHUP
	set.addSignal(SIGHUP);
#endif
	// Operation Error Signals
#ifdef HAVE_SIGPIPE
	set.addSignal(SIGPIPE);
#endif
#ifdef HAVE_SIGLOST
	set.addSignal(SIGLOST);
#endif
	// Miscellaneous Signals
#ifdef HAVE_SIGTTIN
	set.addSignal(SIGTTIN);
#endif
#ifdef HAVE_SIGTTOU
	set.addSignal(SIGTTOU);
#endif
#ifdef HAVE_SIGVTALRM
	set.addSignal(SIGVTALRM);
#endif
#ifdef HAVE_SIGPROF
	set.addSignal(SIGPROF);
#endif
#ifdef HAVE_SIGIO
	set.addSignal(SIGIO);
#endif
#ifdef HAVE_SIGPOLL
	set.addSignal(SIGPOLL);
#endif
	signalmanager::ignoreSignals(&set);

	return sighandler;
}
